from erlei.underscore import underscore as _
from erlei.underscore import uunderscore as __
from erlei.pipe import pipe

__all__ = [
    '_',
    '__',
    'pipe'
]

__version__ = "1.0.0"
name = "erlei"